@@ -11,3 +11,9 @@ pub mod fast_hyper_context;
 
 #[cfg(feature = "hyper_server")]
 pub mod hyper_request;
+
+#[cfg(feature = "actix_server")]
+pub mod basic_actix_context;
+
+#[cfg(feature = "actix_server")]
+pub mod actix_request;
